class MtController < ApplicationController
  unloadable
 
  before_filter :find_project_by_project_id, :authorize, :get_trackers
  menu_item :traceability_matrix
  
  helper :issues
  helper :projects
  helper :queries
  include QueriesHelper
  helper :sort
  include SortHelper

  attr_accessor :issue_rows
  attr_accessor :issue_cols 
  attr_accessor :tracker_rows
  attr_accessor :tracker_cols
  attr_accessor :issue_pairs
  attr_accessor :not_seen_issue_cols
    
  def index
    @plugin = Redmine::Plugin.find(:traceability_matrix)
    @settings = Setting.send "plugin_#{@plugin.id}"
    build_list_of_issues
  end

  def init_macro_context(project)
    @project = project
  end
  
  def build_list_of_issues 
    @issue_rows = @query_rows.issues().sort {|issue_a, issue_b| issue_b.id <=> issue_a.id}
    @issue_cols = @query_cols.issues().sort {|issue_a, issue_b| issue_b.id <=> issue_a.id}
    relations = IssueRelation.find(:all,
                                   :joins => 'INNER JOIN issues issue_from ON issue_from.id = issue_relations.issue_from_id ' +
                                             'INNER JOIN issues issue_to ON issue_to.id = issue_relations.issue_to_id ',
                                   :include => [ :issue_from, :issue_to ],
                                   :conditions => [ 'issue_from.project_id IN ' + @query_rows.project_statement.delete("projects.id IN") +
                                                    'AND issue_to.project_id IN ' + @query_cols.project_statement.delete("projects.id IN") +
                                                    'AND ((issue_from.tracker_id = :trows AND issue_to.tracker_id = :tcols) ' +
                                                         'OR (issue_from.tracker_id = :tcols AND issue_to.tracker_id = :trows))',
                                                    { :trows => @tracker_rows.id, :tcols => @tracker_cols.id } ])

    @not_seen_issue_cols = @issue_cols.dup
    @issue_pairs = {}
    relations.each do |relation|
      if relation.issue_from.tracker_id == tracker_rows.id
        @issue_pairs[relation.issue_from] ||= {}
        @issue_pairs[relation.issue_from][relation.issue_to] ||= []
        @issue_pairs[relation.issue_from][relation.issue_to] << true
        @not_seen_issue_cols.delete relation.issue_to
      else
        @issue_pairs[relation.issue_to] ||= {}
        @issue_pairs[relation.issue_to][relation.issue_from] ||= []
        @issue_pairs[relation.issue_to][relation.issue_from] << true
        @not_seen_issue_cols.delete relation.issue_from
      end
    end

    return 
    
  end
  
  def get_trackers(query_rows_id=Setting.plugin_traceability_matrix['tracker0'], query_cols_id=Setting.plugin_traceability_matrix['tracker1'])
    @tracker_rows = nil
    @query_rows = Query.find(query_rows_id)
    @query_rows.project = @project
    @query_rows.filters.each do |name, options|
      if (name == "tracker_id")
        tracker_id = options[:values].first
         @tracker_rows = Tracker.find(tracker_id)
      end
      
    end
    @tracker_cols = nil
    @query_cols = Query.find(query_cols_id)
    @query_cols.project = @project
    @query_cols.filters.each do |name, options|
      if (name == "tracker_id")
        tracker_id_ = options[:values].first
         @tracker_cols = Tracker.find(tracker_id_)
      end
    end   

  rescue ActiveRecord::RecordNotFound
    flash[:error] = l(:'traceability_matrix.setup')
    render
  end
end

